------------------------------------
-- HPCustomer Beta (3/11/2020)
------------------------------------

-- Create saved global vars
_G.HPCustomerDB = _G.HPCustomerDB or {}

-- Create global vars
_G.HPCustomerGUI = _G.HPCustomerGUI or {}
_G.HPCustomerMain = _G.HPCustomerMain or {}

-- Create local vars
local DB -- assigned during ADDON_LOADED
local Main = _G.HPCustomerMain
local GUI = _G.HPCustomerGUI

-- Hardcoded config vars
Main.BEST_MANAGER_NAME = "Hpmanager"
local SUPPORTED_SERVER = "Rattlegore"
local SUPPORTED_FACTION = "Horde"

-- Register for events
local HPCustomerEvent = CreateFrame("FRAME")
HPCustomerEvent:RegisterEvent("ADDON_LOADED")
HPCustomerEvent:RegisterEvent("CHAT_MSG_ADDON")

-- Main event handler function
local function eventHandler(self, event, arg1, arg2, ...)
  if ((event == "ADDON_LOADED") and (arg1 == "HPCustomer")) then
    if ((GetRealmName() == SUPPORTED_SERVER) and (UnitFactionGroup("player") == SUPPORTED_FACTION)) then
      print("HPCustomer Addon Loading - /hp to open")
      Main:Init()
    else
      HPCustomerEvent:UnregisterEvent("CHAT_MSG_ADDON")
    end
  elseif ((event == "CHAT_MSG_ADDON") and (arg1 == "HPSummons")) then
    local msg, _, sender = arg2, ...
    sender = Main:RemoveServerFromName(sender)
    Main:HandleAddonMessage(msg, sender)
  end
end

-- Register event handler
HPCustomerEvent:SetScript("OnEvent", eventHandler)

-- Slash commands
SLASH_HPCUSTOMER1 = "/hpcustomer"
SLASH_HPCUSTOMER2 = "/hp"
SlashCmdList["HPCUSTOMER"] = function(msg)
  if ((msg == nil) or (msg == "")) then
    Main:UpdateManagerStatus()
    GUI:Create()
    GUI:Show()
  else
    local cmd = {strsplit(" ", msg)}
    local used = false
    if ((#cmd > 1) and (cmd[1] == "remove")) then
      if ((#cmd > 2) and (cmd[2] == "character")) then
        Main:RemoveCharacter(Main:PlayerName(cmd[3]))
        used = true
      elseif ((#cmd > 2) and (cmd[2] == "manager")) then
        Main:RemoveManager(Main:PlayerName(cmd[3]))
        used = true
      end
    elseif ((#cmd > 1) and (cmd[1] == "add")) then
      if ((#cmd > 2) and (cmd[2] == "manager")) then
        Main:AddManager(Main:PlayerName(cmd[3]))
        used = true
      end
    elseif ((#cmd > 1) and (cmd[1] == "blacklist")) then
      if ((#cmd > 2) and (cmd[2] == "character")) then
        Main:BlacklistCharacter(Main:PlayerName(cmd[3]))
        used = true
      end
    elseif ((#cmd > 1) and (cmd[1] == "unblacklist")) then
      if ((#cmd > 2) and (cmd[2] == "character")) then
        Main:UnblacklistCharacter(Main:PlayerName(cmd[3]))
        used = true
      end
    elseif ((#cmd == 1) and (cmd[1] == "debug")) then
      Main.debug = true
      used = true
    end

    if (not used) then
      Main:PrintUsage()
    end
  end
end

-- Init functions

function Main:Init()
  Main:CopyDBDefaults(_G.HPCustomerDB, _G.HPCustomerDBDefaults)
  DB = _G.HPCustomerDB.Main
  C_ChatInfo.RegisterAddonMessagePrefix("HPSummons")
  GUI:Init()
  Main:AddCharacter()
  Main:LoginSummonRequestCheck()

  Main.managerOnBreak = {}
  Main.managerBreakEndTime = {}
  Main.locationList = {}
  Main.lineLength = {}
end

function Main:CopyDBDefaults(db, defaults)
  for k,v in pairs(defaults) do
    if (type(v) == "table") then
      if (type(db[k]) ~= "table") then db[k] = {} end
      HPCustomerMain:CopyDBDefaults(db[k], v)
    elseif (type(v) ~= type(db[k])) then
      db[k] = v
    end
  end
end

function Main:AddCharacter()
  local chars = DB.characterList
  local blacklist = DB.blacklist
  local name = UnitName("player")
  local matched = false

  for i=1,#blacklist do
    if (blacklist[i] == name) then
      matched = true
      break
    end
  end
  if (not matched) then
    for i=1,#chars do
      if (chars[i] == name) then
        matched = true
        break
      end
    end
  end
  if (not matched) then
    table.insert(chars, name)
  end
end

function Main:RemoveCharacter(name)
  local chars = DB.characterList
  local found = false
  for i=1,#chars do
    if (chars[i] == name) then
      table.remove(chars, i)
      print("Removed character: "..name)
      found = true
      break
    end
  end
  if (not found) then
    print("Tried to remove nonexistent character: "..name)
    print("Check spelling and capitalization!")
  end
end

function Main:BlacklistCharacter(name)
  local chars = DB.characterList
  local found = false
  for i=1,#chars do
    if (chars[i] == name) then
      table.remove(chars, i)
      table.insert(DB.blacklist, name)
      print("Blacklisted character: "..name)
      found = true
      break
    end
  end
  if (not found) then
    print("Tried to blacklist nonexistent character: "..name)
    print("Check spelling and capitalization!")
  end
end

function Main:UnblacklistCharacter(name)
  local chars = DB.characterList
  local blacklist = DB.blacklist
  local found = false
  for i=1,#blacklist do
    if (blacklist[i] == name) then
      table.remove(blacklist, i)
      print("Unblacklisted character: "..name)
    end
  end
  if (not found) then
    print("Tried to unblacklist nonexistent character: "..name)
    print("Check spelling and capitalization!")
  end
end

function Main:LoginSummonRequestCheck()
  if (DB.summonRequested) then
    if ((time() - 1800) > DB.summonRequestTime) then
      DB.summonRequested = false
    elseif (DB.inviteReady and (UnitName("player") == DB.summonTarget)) then
      GUI:ShowPopupFrame("LOGIN_INVITE_READY")
      DB.inviteReady = false
    else
      GUI:ShowPopupFrame("SUMMON_REQUESTED_PRIOR_TO_LOGIN")
    end
  else
    DB.inviteReady = false
  end
end

-- Field update functions

function Main:UpdateManagerStatus()
  if (DB.summonRequested) then
    Main:RequestETAInfo()
    Main:StartETATicker()
  else
    Main.availableManagers = {}
    GUI:ClearManagerSelect()

    local managers = DB.managerList
    for i=1,#managers do
      Main.managerOnBreak[managers[i]] = false
      Main.locationList[managers[i]] = {}
      Main.lineLength[managers[i]] = "Unknown"
      C_ChatInfo.SendAddonMessage("HPSummons", "!checkstatus", "WHISPER", managers[i])
    end
  end
end

function Main:CheckActiveManager()
  DB.lastActiveCheckTime = time()
end

function Main:AddManager(name)
  local managers = DB.managerList
  local found = false
  for i=1,#managers do
    if (managers[i] == name) then
      found = true
      break
    end
  end
  if (not found) then
    table.insert(managers, name)
  end
end

function Main:RemoveManager(name)
  local managers = DB.managerList
  for i=1,#managers do
    if (managers[i] == name) then
      table.remove(managers, i)
      break
    end
  end
end

function Main:RequestETAInfo()
  if (DB.summonManager) then
    C_ChatInfo.SendAddonMessage("HPSummons", "!eta", "WHISPER", DB.summonManager)
  end
end

function Main:HandleGUIHide()
  Main:CancelETATicker()
end

-- Ticker functions

function Main:StartETATicker()
  if (Main.etaTicker == nil) then
    Main.etaTicker = C_Timer.NewTicker(30, function()
      Main:RequestETAInfo()
    end)
  end
end

function Main:CancelETATicker()
  if (Main.etaTicker ~= nil) then
    Main.etaTicker:Cancel()
    Main.etaTicker = nil
  end
end

-- Request functions

function Main:RequestSummon(manager, character, location)
  if (location == "") then
    GUI:ShowErrorFrame("Please fill in a location.")
  else

    DB.summonRequested = true
    DB.summonManager = manager
    DB.summonDestination = location
    if (character == "Current Character") then
      DB.summonTarget = UnitName("player")
    else
      DB.summonTarget = character
    end
    DB.summonRequestTime = time()

    C_ChatInfo.SendAddonMessage("HPSummons", "!request_"..DB.summonTarget.."_"..DB.summonDestination, "WHISPER", DB.summonManager)

    Main:RequestETAInfo()
    GUI:Update()
    Main:StartETATicker()
  end
end

function Main:CancelSummon(noMessage)
  DB.summonRequested = false
  GUI:Update()
  Main:CancelETATicker()

  if (noMessage == nil) then
    C_ChatInfo.SendAddonMessage("HPSummons", "!cancel_"..DB.summonTarget.."_"..DB.summonDestination, "WHISPER", DB.summonManager)
  end
end

function Main:LoginConfirmSummon()
  C_ChatInfo.SendAddonMessage("HPSummons", "!loginconfirm_"..DB.summonTarget.."_"..DB.summonDestination, "WHISPER", DB.summonManager)
end

function Main:SetManagerBreak(sender, time)
  Main.managerOnBreak[sender] = true
  Main.managerBreakEndTime[sender] = time
  GUI:Update()
end

function Main:IsManager(name)
  for i=1,#DB.managerList do
    if (name == DB.managerList[i]) then
      return true
    end
  end
  return false
end

-- Group functions

function Main:HandleInviteFail(reason)
  local reason = string.upper(reason)
  GUI:ShowPopupFrame("INVITE_FAIL_"..reason)
end

function Main:RequestInvite()
  C_ChatInfo.SendAddonMessage("HPSummons", "!invite", "WHISPER", DB.summonManager)
end

-- Message functions

function Main:HandleAddonMessage(msg, sender)
  if (Main.debug) then
    print("HPCustomer got "..msg)
  end
  if (Main:IsManager(sender) or (msg:find("!arrived") == 1)) then
    local m = {strsplit("_", msg)}
    if (m[1] == "!locrsp") then
      for i=2,#m do
        if (tonumber(m[i]) ~= nil) then
          Main.lineLength[sender] = tonumber(m[i])
        else
          table.insert(Main.locationList[sender], m[i])
        end
      end
      if (sender == Main.BEST_MANAGER_NAME) then
        table.insert(Main.availableManagers, 1, sender)
        GUI:SetManagerSelect(sender)
      else
        table.insert(Main.availableManagers, sender)
      end
      GUI:Update()
    elseif (m[1] == "!etarsp") then
      Main.etaLineSpot = m[2]
      Main.etaLocOrder = {}
      local i = 3
      while (i < #m) do
        table.insert(Main.etaLocOrder, {m[i], m[i+1]})
        i = i + 2
      end
      GUI:ETAUpdate()
    elseif (m[1] == "!etarspcont") then
      local i = 2
      while (i < #m) do
        table.insert(Main.etaLocOrder, {m[i], m[i+1]})
        i = i + 2
      end
      GUI:ETAUpdate()
    elseif (m[1] == "!cancelrequest") then
      if (m[2] == "break") then
        Main:CancelSummon(true)
        Main:SetManagerBreak(sender, m[3])
      elseif (m[2] == "location") then
        Main.locationList[sender] = {}
        GUI:ClearLocationSelect()
        Main:CancelSummon(true)
        GUI:ShowErrorFrame("Sorry, "..DB.summonManager.." recently disabled this location.")
        Main:UpdateManagerStatus()
      end
    elseif ((m[1] == "!manager") and (sender == Main.BEST_MANAGER_NAME)) then
      if (m[2] == "add") then
        Main:AddManager(m[3])
      elseif (m[2] == "remove") then
        Main:RemoveManager(m[3])
      end
    elseif (m[1] == "!break") then
      Main:SetManagerBreak(sender, m[2])
      table.insert(Main.availableManagers, sender)
      GUI:Update()
    elseif (m[1] == "!inviteready") then
      DB.inviteReady = true
    elseif (m[1] == "!invitefail") then
      Main:HandleInviteFail(m[2])
    elseif (m[1] == "!arrived") then
      DB.summonRequested = false
      GUI:Update()
      Main:CancelETATicker()
    elseif (m[1] == "!version") then
      if (tonumber(m[2]) > 1) then
        print("HPCustomer addon needs an update! Ask "..Main.BEST_MANAGER_NAME.." for more info.")
      end
    end
  end
end

-- Print functions

function Main:PrintUsage()
  print("Printing commands for the HPCustomer addon:")
  print("  /hp -- open gui")
  print("  /hp remove character Charactername -- remove Charactername from the list of characters to get summoned (for deleted characters)")
  print("  /hp blacklist character Charactername -- remove Charactername from the list of characters to get summoned (for existing characters)")
  print("  /hp unblacklist character Charactername -- undo a previous blacklist command")
end

-- Helper functions

function Main:RemoveServerFromName(name)
  local i = name:find("-")
  if (i) then
    name = string.sub(name, 1, i-1)
  end
  return name
end

function Main:PlayerName(name)
  local firstchar
  if (string.byte(name) > 191) then
    firstchar = 2
  else
    firstchar = 1
  end
  local nameuc = string.upper(string.sub(name, 1, firstchar))
  local namelc = string.lower(string.sub(name, firstchar + 1))
  return (nameuc..namelc)
end

